import React, { useState, useEffect } from "react";
import {
  LockOutlined,
  UserOutlined,
  LogoutOutlined,
  SearchOutlined,
} from "@ant-design/icons";
import { Layout, Menu, theme, Dropdown, Tooltip, Avatar, Form, Modal, message } from "antd";
import { ProForm, ProFormText } from '@ant-design/pro-components';
import { history, useModel, useAliveController } from "umi";
import { doFetch } from "@/utils/doFetch";
const { Header } = Layout;
import { fakeAccountLoginOut } from "@/services/login";
import { useRequest } from 'ahooks';
import AES from "crypto-js/aes";
import ECB from "crypto-js/mode-ecb";
import Pkcs7 from "crypto-js/pad-pkcs7";
import Utf8 from "crypto-js/enc-utf8";
import dayjs from "dayjs";

function Headers({ children }) {
  const { initialState: { currentUser, routes }, setInitialState } = useModel("@@initialState");
  const { dropScope } = useAliveController();

  const [visible, cv] = useState(false);
  const {
    token: { colorBgContainer, colorTextBase },
  } = theme.useToken();
  const { runAsync, loading } = useRequest(doFetch, {
    manual: true,
    formatResult: (res) => res,
    onSuccess: (result, params) => {
      if (result.code === '0000') {
        cv(false);
        message.success('密码修改成功，请重新登录！正在跳转...', 2, logut);
      }
    },
  });
  async function logut() {
    await fakeAccountLoginOut();
    const deleteArr = Object.keys(routes);
    deleteArr.map((it) => {
      dropScope(it);
    });
    await setInitialState((s) => ({
      ...s,
      routes: {},
      currentUser: undefined,
      routesMenu: undefined,
      buttonPerm: undefined
    }));
    localStorage.clear();
    history.replace("/user/login");
  }
  const items = [
    {
      key: "2",
      label: (
        <span target="_blank" rel="noopener noreferrer">
          修改密码
        </span>
      ),
      icon: <LockOutlined />,
    },
    {
      key: "4",
      danger: true,
      label: "退出登录",
      icon: <LogoutOutlined />,
    },
  ];

  return (
    <Header
      style={{
        padding: 0,
        background: colorBgContainer,
        display: "flex",
        justifyContent: "space-between",
        alignItems: "center",
        height: 48
      }}
    >
      <div style={{ marginLeft: 10 }} className="centerl">
        {children}
      </div>
      <div style={{ marginRight: 24, gap: 12 }} className="centerl">
        <Dropdown
          menu={{
            items,
            onClick: async (e) => {
              switch (e.key) {
                case '2':
                  cv(true);
                  break;
                case '4':
                  await logut();
                  break;
                default:
                  break;
              }
            },
          }}
          placement="bottom"
        >
          <div className="center" style={{ gap: 10, cursor: "default" }}>
            <Avatar style={{ backgroundColor: "#0e5072" }}>{currentUser?.userName?.substr(0, 1)}</Avatar>
            <span className={`anticon`}>
              {currentUser?.userName}
            </span>
          </div>
        </Dropdown>
      </div>
      <Modal
        title="修改密码"
        open={visible}
        onCancel={() => {
          cv(false);
        }}
        destroyOnClose={true}
        maskClosable={false}
        footer={false}
      >
        <ProForm
          submitter={{
            render: (props, doms) => {
              return [...doms];
            },
          }}
          onFinish={async (values) => {
            let newfields = JSON.parse(JSON.stringify(values));
            delete newfields.confirmPassword;

            let timestamp = dayjs().valueOf().toString() + "acb";
            let newtimestamp = AES.encrypt(timestamp, Utf8.parse("NANGAODEAESKEY--"), {
              mode: ECB,
              padding: Pkcs7,
            }).toString();
            let newPassword = AES.encrypt(values.newPassword, Utf8.parse(timestamp), {
              mode: ECB,
              padding: Pkcs7,
            }).toString(),
              password = AES.encrypt(values.password, Utf8.parse(timestamp), {
                mode: ECB,
                padding: Pkcs7,
              }).toString();

            let postData = {
              password,
              newPassword,
              encryptKey: newtimestamp,
            };
            await runAsync({ url: "/ta_wms_auth/sysUser/changePassword", params: { ...postData } });
          }}
        >
          <ProFormText
            width="lg"
            name="password"
            label="原密码"
            placeholder="请输入原密码"
            formItemProps={{
              rules: [
                {
                  required: true,
                  message: '请输入原密码！',
                },
              ],
            }}
            valueType="password"
          />
          <ProFormText
            width="lg"
            name="newPassword"
            label="新密码"
            placeholder="请输入新密码"
            formItemProps={{
              rules: [
                {
                  required: true,
                  message: '请输入新密码！',
                },
                {
                  pattern: new RegExp(/^[a-zA-Z\d]{6,32}$/),
                  message: '密码应当为6~32位英文字母和数字组合',
                },
              ],
            }}
            valueType="password"
          />
          <ProFormText
            name="encryptKey"
            width="lg"
            label="确认密码"
            placeholder="请确认密码"
            formItemProps={{
              rules: [
                {
                  required: true,
                  message: '请再次输入新密码！',
                },
                ({ getFieldValue }) => ({
                  validator(_, value) {
                    if (!value || getFieldValue('newPassword') === value) {
                      return Promise.resolve();
                    }
                    return Promise.reject(new Error('两次密码不一致！'));
                  },
                }),
              ],
            }}
            valueType="password"
          />
        </ProForm>
      </Modal>
    </Header>
  );
}

export default Headers;
