import React from "react";
import { useLocation, history, useAliveController } from "umi";
import { Swiper, SwiperSlide } from "swiper/react";
import { theme, Dropdown } from "antd";
import { CloseOutlined } from "@ant-design/icons";
import "swiper/css";
import "swiper/css/pagination";
import Slide from "react-reveal/Slide";
import "../index.less";

const RouteItem = ({ iscur, curoute, list, setList }) => {
  const { pathname } = useLocation();
  const { getCachingNodes, dropScope, refreshScope } = useAliveController();
  const {
    token: { colorBgContainer, colorTextBase, colorPrimary },
  } = theme.useToken();
  const cachingNodes = getCachingNodes();

  function refreshCur() {
    refreshScope(curoute.key);
  }

  function closeCur() {
    dropScope(curoute.key);
    //清空操作
    setList((s) => {
      let news = s ? JSON.parse(JSON.stringify(s)) : {};
      const nextIndex = Object.keys(news).indexOf(curoute.key);
      delete news[curoute.key];
      if (pathname === curoute.key) {
        const nextPath =
          Object.keys(news)?.length - 1 < nextIndex
            ? Object.keys(news)[Object.keys(news)?.length - 1]
            : Object.keys(news)[nextIndex];
        history.push(nextPath);
      }
      return news;
    });
  }

  function closeOther() {
    //清空操作
    if (pathname !== curoute.key) {
      history.push(curoute.key);
    }

    setList((s) => {
      let news = JSON.parse(JSON.stringify(s));
      const deleteArr = Object.keys(news).filter((it) => it !== curoute.key);

      deleteArr.map((it) => {
        dropScope(it);
      });

      return {
        [curoute.key]: curoute,
      };
    });
  }

  function closeRight() {
    //清空操作
    setList((s) => {
      let news = JSON.parse(JSON.stringify(s));
      const nextIndex = Object.keys(news).indexOf(curoute.key);
      const deleteKeys = Object.keys(news).filter((it, i) => nextIndex < i);

      if (deleteKeys.includes(pathname)) {
        history.push(curoute.key);
      }
      deleteKeys.map((it) => {
        dropScope(it);
        delete news[it];
      });

      return news;
    });
  }

  return (
    <Slide bottom>
      <Dropdown
        menu={{
          items: [
            {
              key: "1",
              label: "刷新",
            },
            {
              key: "2",
              label: "关闭",
            },
            {
              key: "3",
              label: "关闭其他",
            },
            {
              key: "4",
              label: "关闭右侧",
            },
          ],
          onClick: ({ key }) => {
            switch (key) {
              case "1":
                refreshCur();
                break;
              case "2":
                closeCur();
                break;
              case "3":
                closeOther();
                break;
              case "4":
                closeRight();
                break;

              default:
                break;
            }
          },
        }}
        trigger={["contextMenu"]}
      >
        <div
          className="hoverClose"
          style={{
            background: iscur ?'#0E5072':'#fff',
            position: "relative",
            padding: "8px 10px",
            borderRadius: 8,
            cursor: "pointer",
            color: iscur ? '#e7e2d8' : colorTextBase,
            fontSize: 12,
            display: "flex",
            alignItems: "center",
            userSelect: "none",
          }}
          onClick={() => {
            history.push(curoute?.key);
          }}
        >
          <span>{curoute?.label}</span>

          {Object.keys(list).length > 1 && (
            <CloseOutlined
              style={{ fontSize: 10 }}
              className="close"
              onClick={(e) => {
                e.stopPropagation();
                closeCur();
              }}
            ></CloseOutlined>
          )}
        </div>
      </Dropdown>
    </Slide>
  );
};

function NavBar({ list = {}, setList }) {
  const { pathname } = useLocation();
  const {
    token: { colorBgContainer, colorTextBase },
  } = theme.useToken();

  return (
    <div style={{ display: "flex", width: "100%",marginTop:10,paddingLeft:16 }}>
      <Swiper slidesPerView={"auto"} spaceBetween={8} style={{ width: "100%" }}>
        {Object.entries(list).map(([curkey, curoute]) => {
          const iscur = curkey === pathname;
          return (
            <SwiperSlide
              key={curkey}
              style={{
                color: colorTextBase,
                display: "inline",
                width: "auto",
              }}
            >
              <RouteItem
                curoute={curoute}
                iscur={iscur}
                list={list}
                setList={setList}
              ></RouteItem>
            </SwiperSlide>
          );
        })}
      </Swiper>
    </div>
  );
}

export default NavBar;
