import * as AntdIcons from "@ant-design/icons";
import { Layout, Menu, theme, Drawer, Input, Dropdown, Avatar } from "antd";
import React, { useState, useRef, useEffect } from "react";
import {
  Outlet,
  useLocation,
  history,
  useOutletContext,
  useModel,
  useOutlet,
  KeepAlive,
  Link
} from "umi";
import Headers from "./components/Header";
import routes from "../../config/routes";
import defaultSettings from "../../config/defaultSettings";
import { Scrollbars } from "react-custom-scrollbars";
import { useRequest, useUpdate } from "ahooks";
import NavBar from "./components/NavBar";
import pinyin from "pinyin";
import "./index.less";
import { useMemo } from "react";

const { MenuFoldOutlined, MenuUnfoldOutlined, SearchOutlined } = AntdIcons;

function capitalizeFirstLetter(str) {
  return str
    .replace(/-([a-z])/g, (match, letter) => letter.toUpperCase())
    .replace(/\b\w/g, (match) => match.toUpperCase());
}

//format routes
const getTreeParents = (treeData, type) => {
  let res = treeData;
  if (type === "local") {
    res = treeData
      ?.filter((it) => it.name)
      ?.filter((it) => !it.hideInMenu)
      .map((item) => {
        const Icon =
          AntdIcons[capitalizeFirstLetter(item?.icon ?? "file") + "Outlined"];
        if (item.routes && item.routes.length) {
          item = {
            key: item.path,
            icon: item.icon ? <Icon></Icon> : null,
            children: getTreeParents(item.routes, type),
            label: item.name,
          };
        } else {
          item = {
            key: item.path,
            icon: item.icon ? <Icon></Icon> : null,
            children: item.routes,
            label: item.name,
          };
        }
        return item;
      });
  } else {
    res = treeData?.map((item) => {
      const Icon =
        AntdIcons[capitalizeFirstLetter(item?.icon ?? "file") + "Outlined"];
      if (item.routes && item.routes.length) {
        item = {
          key: item.path,
          icon: item.icon && Icon ? <Icon></Icon> : null,
          children: getTreeParents(item.routes, type),
          label: item.name,
        };
      } else {
        item = {
          key: item.path,
          icon: item.icon && Icon ? <Icon></Icon> : null,
          children: item.routes,
          label: item.name,
        };
      }
      return item;
    });
  }
  return res;
};

//默认展开key
function findParentKey(tree, key, parentKey = null) {
  for (const node of tree) {
    if (node.key === key) {
      return parentKey; // 找到匹配的节点，返回父节点的键值
    }
    if (node.children && node.children.length > 0) {
      const result = findParentKey(node.children, key, node.key);
      if (result !== null) {
        return result; // 在子树中找到匹配的节点，返回结果
      }
    }
  }
  return null; // 未找到匹配的节点
}

//找到当前节点
function findNodeById(tree, targetId) {
  let result = null;

  function recursiveSearch(node) {
    if (node.key === targetId) {
      result = node;
      return;
    }

    if (node.children && node.children.length > 0) {
      for (const child of node.children) {
        recursiveSearch(child);
        if (result) {
          return;
        }
      }
    }
  }

  // 从树的根节点开始搜索
  for (const node of tree) {
    recursiveSearch(node);
    if (result) {
      break;
    }
  }
  result={
    ...result,
    icon:null
  };
  return result;
}

// 过滤函数
function filterByPinyin(data, keyword) {
  const pinyinres = pinyin(data, {
    style: "normal", // 使用普通风格的拼音
  }).join("");

  // 检查拼音是否包含关键字
  return pinyinres.includes(keyword);
}

// 根据字符串筛选出匹配的项的函数
function filterTree(trees, searchString) {
  const result = [];

  function searchNode(node, searchString) {
    if (node.label.includes(searchString) || filterByPinyin(node.label, searchString)) {
      result.push(node);
    }

    if (node.children) {
      for (const child of node.children) {
        searchNode(child, searchString);
      }
    }
  }

  for (const tree of trees) {
    searchNode(tree, searchString);
  }

  return result;
}
function getLeafNode(data){
    let leafArrs = [];
    function getNode(node){
      if(node.children){
        node.children?.map(it=>{
          getNode(it);
        })
      }else{
        leafArrs.push(node);
      }
    }
    data.map(it=>{
      getNode(it);
    });
    return leafArrs;
}

const { Content, Sider } = Layout;
const curroutes = routes
  ?.filter((it) => it.path === "/")?.[0]
  ?.routes?.filter((it) => it.name && !it.hideInMenu);

const ifs = true;
const Dashboard = () => {
  const [search, setSearch] = useState(null);
  const [collapsed, setCollapsed] = useState(false);
  const {
    token: { colorTextBase, colorBgBase ,colorPrimary},
  } = theme.useToken();
  const { initialState: { routesMenu=[], routes={} }, setInitialState } = useModel("@@initialState");

  const items = useMemo(() => {
    let newmenu = [
      {
        name: '首页',
        path: '/home',
        parentKey: '0',
        key: '000000',
        icon: 'home',
      },
      ...routesMenu
    ];
    const item = ifs
      ? getTreeParents(newmenu ?? [])
      : getTreeParents(curroutes, "local");
      if(!search){
        return item;
      }else{
        let leafs = getLeafNode(item);
        return filterTree(leafs, search)
      }
  }, [routesMenu, search, ifs]);
  const { pathname } = useLocation();

  const { loading } = useRequest(async () => {
    let res = await new Promise((resolve) => {
      setTimeout(() => resolve(1), 500);
    });
    return res;
  }, {});
  const MenuDOM = () => {
    return (
      <div
        style={{
          backgroundColor: defaultSettings.primaryColor,
          height: "calc(100%)",
          color: colorTextBase,
          display: "flex",
          flexDirection: "column",
          borderRadius: "10px",
        }}
      >
        <div className='layoutlogo' id="logo" style={{ textAlign: !collapsed ? 'left' : 'center' }}>
          <Link to="/home">
            <img src={defaultSettings.logo} alt="logo" />
            {!collapsed && <h1 style={{ fontSize: 12 }}>{defaultSettings.name}</h1>}
          </Link>
        </div>
        {
          !collapsed && <div
            style={{ height: 48, gap: 12, padding: "18px 12px 8px 12px" }}
            className="center"
          >
            <div
              className="center"
              style={{
                height: 36,
                backgroundColor: "#F0F0F0",
                flex: 1,
                overflow: "hidden",
                borderRadius: 8,
              }}
            >
              <Input
                bordered={false}
                prefix={<SearchOutlined />}
                style={{ outline: "none" }}
                value={search}
                onChange={(e) => {
                  setSearch(e.target.value);
                }}
                placeholder="菜单筛选"
              ></Input>
            </div>
          </div>
        }
        <div style={{ flex: 1 }}>
          <Scrollbars
            thumbMinSize={36}
            autoHide
            style={{
              width: "100%",
              height: "100%",
            }}
            hideTracksWhenNotNeeded
          >
            <div style={{ padding: 6 }}>
              {!loading && (
                <Menu
                  theme={"dark"}
                  mode="inline"
                  defaultOpenKeys={[findParentKey(items, pathname)]}
                  selectedKeys={[pathname]}
                  items={items}
                  style={{
                    background: "transparent",
                  }}
                  onClick={(e) => {
                    history.push(e.key);
                  }}
                  inlineIndent={12}
                />
              )}
            </div>
          </Scrollbars>
        </div>
      </div>
    );
  };

  const setRoutes = (val) => {
    if (typeof val === "function") {
      setInitialState((s) => {
        return {
          ...s,
          routes: val?.(s.routes),
        };
      });
    } else {
      setInitialState((s) => ({
        ...s,
        routes: val,
      }));
    }
  };

  useEffect(() => {
    if (routes && !Object.keys(routes).includes(pathname) && pathname !== "/") {
      let curroute = findNodeById(items, pathname);
      if (curroute?.children?.length > 0) {
        return;
      }
      setRoutes((s) => {
        return {
          ...s,
          [pathname]: findNodeById(items, pathname),
        };
      });
    }
  }, [pathname]);
  return (
    <Layout hasSider className="contained">
      <Sider
        breakpoint="lg"
        onBreakpoint={(broken) => {
          setCollapsed(broken);
        }}
        style={{
          overflow: "hidden",
          backgroundColor: "transparent",
        }}
        trigger={null}
        collapsible
        collapsed={collapsed}
        collapsedWidth={66}
      >
        {MenuDOM()}
      </Sider>
      <Layout>
        <Content
          style={{
            flex: 1,
            overflow: "auto",
            padding: 0,
            color: colorTextBase,
            // background: colorBgBase,
            borderRadius: 10
          }}
        >
          <Headers>
            <div className="sorts">
              {React.createElement(
                collapsed ? MenuUnfoldOutlined : MenuFoldOutlined,
                {
                  className: "trigger",
                  style: { color: colorTextBase },
                  onClick: () => setCollapsed(!collapsed),
                }
              )}
            </div>
          </Headers>
          <NavBar list={routes} setList={setRoutes}></NavBar>
          <div style={{ height: 'calc(100% - 110px)', width: '100%', margin: '10px 0 10px 0', background: '#fff' }}>
            <Scrollbars
              thumbMinSize={36}
              autoHide
              style={{
                width: "100%",
                height: "100%",
              }}
              hideTracksWhenNotNeeded
            >
              <Outlet></Outlet>
            </Scrollbars>
          </div>
        </Content>
      </Layout>
    </Layout>
  );
};
export default Dashboard;
